% P = KNNPrecision(trainZ,testZ,K,Y) K-nearest neighbors precision
%
% In:
%   trainZ: NxL binary matrix containing binary codes for training set.
%   testZ: MxL binary matrix containing binary codes for test set.
%   K: number of neighbors, or a list of neighbors.
%   Y: MxN affinty matrix containing the neighborhood information of the
%      test points (ground truth). The (i,j)th element of Y is 1 if test
%      point i and training point j are similar.
% Out:
%   P: K-nearest neighbor precision or list of precisions (if K is a list).

% Copyright (c) 2016 by Ramin Raziperchikolaei and Miguel A. Carreira-Perpinan

function P = KNNPrecision(trainZ,testZ,K,Y)

[N,~] = size(testZ); P = zeros(size(K));
for i = 1:N
  point = testZ(i,:); yidx = find(Y(i,:)==1);
  dist = sum(bsxfun(@xor,trainZ,point),2);
  [~,idx] = sort(dist);
  for j = 1:numel(K)
    idx1 = idx(1:K(j));    
    c = intersect(idx1,yidx);
    retrievedGoodPairs = numel(c);
    P(j) = P(j) + retrievedGoodPairs/K(j);
  end
end
P = P/N;

end

